% Read the measured input/output speaker cab data
tmp = audioread('sine_measurement.wav');

% y is the microphone's recording, x is the sine sweep track
y = tmp(:,2)';
x = tmp(:,1)';

% Linear algebra magic
A = [x' [0 x(1:end-1)]' [0 0 x(1:end-2)]' [0 0 0 x(1:end-3)]' [0 0 0 0 x(1:end-4)]' [0 0 0 0 0 x(1:end-5)]' [0 0 0 0 0 0 x(1:end-6)]' [0 0 0 0 0 0 0 x(1:end-7)]' [0 0 0 0 0 0 0 0 x(1:end-8)]' [0 y(1:end-1)]' [0 0 y(1:end-2)]' [0 0 0 y(1:end-3)]' [0 0 0 0 y(1:end-4)]' [0 0 0 0 0 y(1:end-5)]' [0 0 0 0 0 0 y(1:end-6)]' [0 0 0 0 0 0 0 y(1:end-7)]' [0 0 0 0 0 0 0 0 y(1:end-8)]'];
B = y';

% The \ operator performs least squares regression
X = A\B;

% Extract our coefficients from the solution, X
b = [X(1) X(2) X(3) X(4) X(5) X(6) X(7) X(8) X(9)]; 
a = -[-1 X(10) X(11) X(12) X(13) X(14) X(15) X(16) X(17)];

% Calculate model's frequency response
[H, W] = freqz(b,a, 5000, 48e3);

% Calculate the speaker's frequency reponse
% using deconvolution
Hcab = fft(y)./fft(x);

% Plot everything
f = linspace(0, 48e3, length(y));
figure(2)
hold off
plot(f, 20*log10(abs(Hcab)))
hold on
plot(W, 20*log10(abs(H))+6,'r')
set(gca, "linewidth", 2, "fontsize", 16)
grid on
axis([0 20000 -45 5]);
xlabel('Frequency (Hz)');
ylabel('Amplitude (dB)');
title('Frequency response of speaker (blue) and 8th order regression model (red)');

figure(1)
hold off
semilogx(f, 20*log10(abs(Hcab)))
hold on
semilogx(W, 20*log10(abs(H))+6,'r')
set(gca, "linewidth", 2, "fontsize", 16)
grid on
axis([50 20000 -45 5]);
xlabel('Frequency (Hz)');
ylabel('Amplitude (dB)');
title('Frequency response of speaker (blue) and 8th order regression model (red)');



%yn = filter(b,a,x);
